@@ -1,16 +1,28 @@
+from __future__ import annotations
+
 import json
 import re
 
-from cleo import argument
-from cleo import option
+from pathlib import Path
+from typing import TYPE_CHECKING
+from typing import Any
+from typing import cast
 
-from poetry.factory import Factory
+from cleo.helpers import argument
+from cleo.helpers import option
 
-from .command import Command
+from poetry.config.config import PackageFilterPolicy
+from poetry.config.config import boolean_normalizer
+from poetry.config.config import boolean_validator
+from poetry.config.config import int_normalizer
+from poetry.console.commands.command import Command
+
+
+if TYPE_CHECKING:
+    from poetry.config.config_source import ConfigSource
 
 
 class ConfigCommand(Command):
-
     name = "config"
     description = "Manages configuration settings."
 
@@ -25,7 +37,8 @@ class ConfigCommand(Command):
         option("local", None, "Set/Get from the project's local configuration."),
     ]
 
-    help = """This command allows you to edit the poetry config settings and repositories.
+    help = """\
+This command allows you to edit the poetry config settings and repositories.
 
 To add a repository:
 
@@ -38,45 +51,56 @@ To remove a repository (repo is a short alias for repositories):
     LIST_PROHIBITED_SETTINGS = {"http-basic", "pypi-token"}
 
     @property
-    def unique_config_values(self):
-        from poetry.config.config import boolean_normalizer
-        from poetry.config.config import boolean_validator
-        from poetry.locations import CACHE_DIR
-        from poetry.utils._compat import Path
-
+    def unique_config_values(self) -> dict[str, tuple[Any, Any]]:
         unique_config_values = {
-            "cache-dir": (
-                str,
-                lambda val: str(Path(val)),
-                str(Path(CACHE_DIR) / "virtualenvs"),
+            "cache-dir": (str, lambda val: str(Path(val))),
+            "virtualenvs.create": (boolean_validator, boolean_normalizer),
+            "virtualenvs.in-project": (boolean_validator, boolean_normalizer),
+            "virtualenvs.options.always-copy": (boolean_validator, boolean_normalizer),
+            "virtualenvs.options.system-site-packages": (
+                boolean_validator,
+                boolean_normalizer,
             ),
-            "virtualenvs.create": (boolean_validator, boolean_normalizer, True),
-            "virtualenvs.in-project": (boolean_validator, boolean_normalizer, False),
-            "virtualenvs.path": (
-                str,
-                lambda val: str(Path(val)),
-                str(Path(CACHE_DIR) / "virtualenvs"),
+            "virtualenvs.options.no-pip": (boolean_validator, boolean_normalizer),
+            "virtualenvs.options.no-setuptools": (
+                boolean_validator,
+                boolean_normalizer,
+            ),
+            "virtualenvs.path": (str, lambda val: str(Path(val))),
+            "virtualenvs.prefer-active-python": (boolean_validator, boolean_normalizer),
+            "experimental.new-installer": (boolean_validator, boolean_normalizer),
+            "experimental.system-git-client": (boolean_validator, boolean_normalizer),
+            "installer.modern-installation": (boolean_validator, boolean_normalizer),
+            "installer.parallel": (boolean_validator, boolean_normalizer),
+            "installer.max-workers": (lambda val: int(val) > 0, int_normalizer),
+            "virtualenvs.prompt": (str, lambda val: str(val)),
+            "installer.no-binary": (
+                PackageFilterPolicy.validator,
+                PackageFilterPolicy.normalize,
             ),
         }
 
         return unique_config_values
 
-    def handle(self):
+    def handle(self) -> int:
+        from pathlib import Path
+
+        from poetry.core.pyproject.exceptions import PyProjectException
+        from poetry.core.toml.file import TOMLFile
+
+        from poetry.config.config import Config
         from poetry.config.file_config_source import FileConfigSource
         from poetry.locations import CONFIG_DIR
-        from poetry.utils._compat import Path
-        from poetry.utils._compat import basestring
-        from poetry.utils.toml_file import TomlFile
 
-        config = Factory.create_config(self.io)
-        config_file = TomlFile(Path(CONFIG_DIR) / "config.toml")
+        config = Config.create()
+        config_file = TOMLFile(CONFIG_DIR / "config.toml")
 
         try:
-            local_config_file = TomlFile(self.poetry.file.parent / "poetry.toml")
+            local_config_file = TOMLFile(self.poetry.file.parent / "poetry.toml")
             if local_config_file.exists():
                 config.merge(local_config_file.read())
-        except RuntimeError:
-            local_config_file = TomlFile(Path.cwd() / "poetry.toml")
+        except (RuntimeError, PyProjectException):
+            local_config_file = TOMLFile(Path.cwd() / "poetry.toml")
 
         if self.option("local"):
             config.set_config_source(FileConfigSource(local_config_file))
@@ -100,46 +124,44 @@ To remove a repository (repo is a short alias for repositories):
         # show the value if no value is provided
         if not self.argument("value") and not self.option("unset"):
             m = re.match(r"^repos?(?:itories)?(?:\.(.+))?", self.argument("key"))
+            value: str | dict[str, Any]
             if m:
                 if not m.group(1):
                     value = {}
                     if config.get("repositories") is not None:
                         value = config.get("repositories")
                 else:
-                    repo = config.get("repositories.{}".format(m.group(1)))
+                    repo = config.get(f"repositories.{m.group(1)}")
                     if repo is None:
-                        raise ValueError(
-                            "There is no {} repository defined".format(m.group(1))
-                        )
+                        raise ValueError(f"There is no {m.group(1)} repository defined")
 
                     value = repo
 
                 self.line(str(value))
             else:
-                values = self.unique_config_values
-                if setting_key not in values:
-                    raise ValueError("There is no {} setting.".format(setting_key))
+                if setting_key not in self.unique_config_values:
+                    raise ValueError(f"There is no {setting_key} setting.")
 
                 value = config.get(setting_key)
 
-                if not isinstance(value, basestring):
+                if not isinstance(value, str):
                     value = json.dumps(value)
 
                 self.line(value)
 
             return 0
 
-        values = self.argument("value")
+        values: list[str] = self.argument("value")
 
-        unique_config_values = self.unique_config_values
-        if setting_key in unique_config_values:
+        if setting_key in self.unique_config_values:
             if self.option("unset"):
-                return config.config_source.remove_property(setting_key)
+                config.config_source.remove_property(setting_key)
+                return 0
 
             return self._handle_single_value(
                 config.config_source,
                 setting_key,
-                unique_config_values[setting_key],
+                self.unique_config_values[setting_key],
                 values,
             )
 
@@ -150,24 +172,18 @@ To remove a repository (repo is a short alias for repositories):
                 raise ValueError("You cannot remove the [repositories] section")
 
             if self.option("unset"):
-                repo = config.get("repositories.{}".format(m.group(1)))
+                repo = config.get(f"repositories.{m.group(1)}")
                 if repo is None:
-                    raise ValueError(
-                        "There is no {} repository defined".format(m.group(1))
-                    )
+                    raise ValueError(f"There is no {m.group(1)} repository defined")
 
-                config.config_source.remove_property(
-                    "repositories.{}".format(m.group(1))
-                )
+                config.config_source.remove_property(f"repositories.{m.group(1)}")
 
                 return 0
 
             if len(values) == 1:
                 url = values[0]
 
-                config.config_source.add_property(
-                    "repositories.{}.url".format(m.group(1)), url
-                )
+                config.config_source.add_property(f"repositories.{m.group(1)}.url", url)
 
                 return 0
 
@@ -198,7 +214,7 @@ To remove a repository (repo is a short alias for repositories):
                 elif len(values) != 2:
                     raise ValueError(
                         "Expected one or two arguments "
-                        "(username, password), got {}".format(len(values))
+                        f"(username, password), got {len(values)}"
                     )
                 else:
                     username = values[0]
@@ -208,7 +224,7 @@ To remove a repository (repo is a short alias for repositories):
             elif m.group(1) == "pypi-token":
                 if len(values) != 1:
                     raise ValueError(
-                        "Expected only one argument (token), got {}".format(len(values))
+                        f"Expected only one argument (token), got {len(values)}"
                     )
 
                 token = values[0]
@@ -218,45 +234,57 @@ To remove a repository (repo is a short alias for repositories):
             return 0
 
         # handle certs
-        m = re.match(
-            r"(?:certificates)\.([^.]+)\.(cert|client-cert)", self.argument("key")
-        )
+        m = re.match(r"certificates\.([^.]+)\.(cert|client-cert)", self.argument("key"))
         if m:
+            repository = m.group(1)
+            key = m.group(2)
+
             if self.option("unset"):
                 config.auth_config_source.remove_property(
-                    "certificates.{}.{}".format(m.group(1), m.group(2))
+                    f"certificates.{repository}.{key}"
                 )
 
                 return 0
 
             if len(values) == 1:
+                new_value: str | bool = values[0]
+
+                if key == "cert" and boolean_validator(values[0]):
+                    new_value = boolean_normalizer(values[0])
+
                 config.auth_config_source.add_property(
-                    "certificates.{}.{}".format(m.group(1), m.group(2)), values[0]
+                    f"certificates.{repository}.{key}", new_value
                 )
             else:
                 raise ValueError("You must pass exactly 1 value")
 
             return 0
 
-        raise ValueError("Setting {} does not exist".format(self.argument("key")))
+        raise ValueError(f"Setting {self.argument('key')} does not exist")
 
-    def _handle_single_value(self, source, key, callbacks, values):
-        validator, normalizer, _ = callbacks
+    def _handle_single_value(
+        self,
+        source: ConfigSource,
+        key: str,
+        callbacks: tuple[Any, Any],
+        values: list[Any],
+    ) -> int:
+        validator, normalizer = callbacks
 
         if len(values) > 1:
             raise RuntimeError("You can only pass one value.")
 
         value = values[0]
         if not validator(value):
-            raise RuntimeError('"{}" is an invalid value for {}'.format(value, key))
+            raise RuntimeError(f'"{value}" is an invalid value for {key}')
 
         source.add_property(key, normalizer(value))
 
         return 0
 
-    def _list_configuration(self, config, raw, k=""):
-        from poetry.utils._compat import basestring
-
+    def _list_configuration(
+        self, config: dict[str, Any], raw: dict[str, Any], k: str = ""
+    ) -> None:
         orig_k = k
         for key, value in sorted(config.items()):
             if k + key in self.LIST_PROHIBITED_SETTINGS:
@@ -265,85 +293,25 @@ To remove a repository (repo is a short alias for repositories):
             raw_val = raw.get(key)
 
             if isinstance(value, dict):
-                k += "{}.".format(key)
+                k += f"{key}."
+                raw_val = cast("dict[str, Any]", raw_val)
                 self._list_configuration(value, raw_val, k=k)
                 k = orig_k
 
                 continue
             elif isinstance(value, list):
-                value = [
+                value = ", ".join(
                     json.dumps(val) if isinstance(val, list) else val for val in value
-                ]
-
-                value = "[{}]".format(", ".join(value))
+                )
+                value = f"[{value}]"
 
             if k.startswith("repositories."):
-                message = "<c1>{}</c1> = <c2>{}</c2>".format(
-                    k + key, json.dumps(raw_val)
-                )
-            elif isinstance(raw_val, basestring) and raw_val != value:
-                message = "<c1>{}</c1> = <c2>{}</c2>  # {}".format(
-                    k + key, json.dumps(raw_val), value
+                message = f"<c1>{k + key}</c1> = <c2>{json.dumps(raw_val)}</c2>"
+            elif isinstance(raw_val, str) and raw_val != value:
+                message = (
+                    f"<c1>{k + key}</c1> = <c2>{json.dumps(raw_val)}</c2>  # {value}"
                 )
             else:
-                message = "<c1>{}</c1> = <c2>{}</c2>".format(k + key, json.dumps(value))
+                message = f"<c1>{k + key}</c1> = <c2>{json.dumps(value)}</c2>"
 
             self.line(message)
-
-    def _list_setting(self, contents, setting=None, k=None, default=None):
-        values = self._get_setting(contents, setting, k, default)
-
-        for value in values:
-            self.line(
-                "<comment>{}</comment> = <info>{}</info>".format(value[0], value[1])
-            )
-
-    def _get_setting(self, contents, setting=None, k=None, default=None):
-        orig_k = k
-
-        if setting and setting.split(".")[0] not in contents:
-            value = json.dumps(default)
-
-            return [((k or "") + setting, value)]
-        else:
-            values = []
-            for key, value in contents.items():
-                if setting and key != setting.split(".")[0]:
-                    continue
-
-                if isinstance(value, dict) or key == "repositories" and k is None:
-                    if k is None:
-                        k = ""
-
-                    k += re.sub(r"^config\.", "", key + ".")
-                    if setting and len(setting) > 1:
-                        setting = ".".join(setting.split(".")[1:])
-
-                    values += self._get_setting(
-                        value, k=k, setting=setting, default=default
-                    )
-                    k = orig_k
-
-                    continue
-
-                if isinstance(value, list):
-                    value = [
-                        json.dumps(val) if isinstance(val, list) else val
-                        for val in value
-                    ]
-
-                    value = "[{}]".format(", ".join(value))
-
-                value = json.dumps(value)
-
-                values.append(((k or "") + key, value))
-
-            return values
-
-    def _get_formatted_value(self, value):
-        if isinstance(value, list):
-            value = [json.dumps(val) if isinstance(val, list) else val for val in value]
-
-            value = "[{}]".format(", ".join(value))
-
-        return json.dumps(value)
