@@ -1,9 +1,10 @@
 from __future__ import annotations
 
+import warnings
+
 from typing import TYPE_CHECKING
 
 from cleo.io.null_io import NullIO
-from packaging.utils import NormalizedName
 from packaging.utils import canonicalize_name
 
 from poetry.installation.executor import Executor
@@ -11,8 +12,8 @@ from poetry.installation.operations import Install
 from poetry.installation.operations import Uninstall
 from poetry.installation.operations import Update
 from poetry.installation.pip_installer import PipInstaller
-from poetry.repositories import Pool
 from poetry.repositories import Repository
+from poetry.repositories import RepositoryPool
 from poetry.repositories.installed_repository import InstalledRepository
 from poetry.repositories.lockfile_repository import LockfileRepository
 from poetry.utils.extras import get_extra_package_names
@@ -21,9 +22,9 @@ from poetry.utils.helpers import pluralize
 
 if TYPE_CHECKING:
     from collections.abc import Iterable
-    from collections.abc import Sequence
 
     from cleo.io.io import IO
+    from packaging.utils import NormalizedName
     from poetry.core.packages.project_package import ProjectPackage
 
     from poetry.config.config import Config
@@ -40,7 +41,7 @@ class Installer:
         env: Env,
         package: ProjectPackage,
         locker: Locker,
-        pool: Pool,
+        pool: RepositoryPool,
         config: Config,
         installed: Repository | None = None,
         executor: Executor | None = None,
@@ -62,7 +63,7 @@ class Installer:
         self._execute_operations = True
         self._lock = False
 
-        self._whitelist: list[str] = []
+        self._whitelist: list[NormalizedName] = []
 
         self._extras: list[NormalizedName] = []
 
@@ -72,7 +73,7 @@ class Installer:
             )
 
         self._executor = executor
-        self._use_executor = False
+        self._use_executor = True
 
         self._installer = self._get_installer()
         if installed is None:
@@ -181,6 +182,14 @@ class Installer:
         return self
 
     def use_executor(self, use_executor: bool = True) -> Installer:
+        warnings.warn(
+            (
+                "Calling use_executor() is deprecated since it's true by default now"
+                " and deactivating it will be removed in a future release."
+            ),
+            DeprecationWarning,
+            stacklevel=2,
+        )
         self._use_executor = use_executor
 
         return self
@@ -202,10 +211,16 @@ class Installer:
             self._io,
         )
 
+        # Always re-solve directory dependencies, otherwise we can't determine
+        # if anything has changed (and the lock file contains an invalid version).
+        use_latest = [
+            p.name for p in locked_repository.packages if p.source_type == "directory"
+        ]
+
         with solver.provider.use_source_root(
             source_root=self._env.path.joinpath("src")
         ):
-            ops = solver.solve(use_latest=[]).calculate_operations()
+            ops = solver.solve(use_latest=use_latest).calculate_operations()
 
         lockfile_repo = LockfileRepository()
         self._populate_lockfile_repo(lockfile_repo, ops)
@@ -295,7 +310,7 @@ class Installer:
             )
 
         # We resolve again by only using the lock file
-        pool = Pool(ignore_repository_names=True)
+        pool = RepositoryPool(ignore_repository_names=True)
 
         # Making a new repo containing the packages
         # newly resolved and the ones from the current lock file
@@ -361,6 +376,14 @@ class Installer:
         if self._use_executor:
             return self._executor.execute(operations)
 
+        self._io.write_error(
+            "<warning>"
+            "Setting `experimental.new-installer` to false is deprecated and"
+            " slated for removal in an upcoming minor release.\n"
+            "(Despite of the setting's name the new installer is not experimental!)"
+            "</warning>"
+        )
+
         if not operations and (self._execute_operations or self._dry_run):
             self._io.write_line("No dependencies to install or update")
 
@@ -473,7 +496,7 @@ class Installer:
         self._installer.remove(operation.package)
 
     def _populate_lockfile_repo(
-        self, repo: LockfileRepository, ops: Sequence[Operation]
+        self, repo: LockfileRepository, ops: Iterable[Operation]
     ) -> None:
         for op in ops:
             if isinstance(op, Uninstall):
@@ -517,7 +540,7 @@ class Installer:
 
         return ops
 
-    def _filter_operations(self, ops: Sequence[Operation], repo: Repository) -> None:
+    def _filter_operations(self, ops: Iterable[Operation], repo: Repository) -> None:
         extra_packages = self._get_extra_packages(repo)
         for op in ops:
             if isinstance(op, Update):
@@ -532,21 +555,12 @@ class Installer:
                 op.skip("Not needed for the current environment")
                 continue
 
-            if self._update:
-                extras = {}
-                for extra, dependencies in self._package.extras.items():
-                    extras[extra] = [dependency.name for dependency in dependencies]
-            else:
-                extras = {}
-                for extra, deps in self._locker.lock_data.get("extras", {}).items():
-                    extras[extra] = [dep.lower() for dep in deps]
-
             # If a package is optional and not requested
             # in any extra we skip it
             if package.optional and package.name not in extra_packages:
                 op.skip("Not required")
 
-    def _get_extra_packages(self, repo: Repository) -> list[str]:
+    def _get_extra_packages(self, repo: Repository) -> set[NormalizedName]:
         """
         Returns all package names required by extras.
 
@@ -564,7 +578,7 @@ class Installer:
                 for extra, dependencies in raw_extras.items()
             }
 
-        return list(get_extra_package_names(repo.packages, extras, self._extras))
+        return get_extra_package_names(repo.packages, extras, self._extras)
 
     def _get_installer(self) -> BaseInstaller:
         return PipInstaller(self._env, self._io, self._pool)
