@@ -211,8 +211,6 @@ class Uploader:
         skip_existing: bool = False,
     ) -> None:
         for file in self.files:
-            # TODO: Check existence
-
             self._upload_file(session, url, file, dry_run, skip_existing)
 
     def _upload_file(
@@ -225,6 +223,9 @@ class Uploader:
     ) -> None:
         from cleo.ui.progress_bar import ProgressBar
 
+        if not file.is_file():
+            raise UploadError(f"Archive ({file}) does not exist")
+
         data = self.post_data(file)
         data.update(
             {
@@ -265,7 +266,7 @@ class Uploader:
                         f" - Uploading <c1>{file.name}</c1> <fg=green>%percent%%</>"
                     )
                     bar.finish()
-                elif resp.status_code == 301:
+                elif 300 <= resp.status_code < 400:
                     if self._io.output.is_decorated():
                         self._io.overwrite(
                             f" - Uploading <c1>{file.name}</c1> <error>FAILED</>"
